using System;
using System.Collections.Generic;
using System.Text;
using System.Data;
using System.Data.SqlTypes;
using System.Data.SqlClient;
using System.Configuration;

using Jefferies.Common.Data.Sql.DataModel;

namespace Jefferies.Common.Data.Sql.DataAccess
{
	public class SqlDataManager
	{
		public const string QuantityFormat = "#,##0";
		public const string PriceFormat = "#,##0.00##";
		public const string FeeFormat = "##0.00";
		public const string DateFormat = "yyyy-MM-dd";

		private int _identity;
		private SqlConnection _sqlConnection;

		/// <summary>
		/// 
		/// </summary>
		/// <param name="connString">Connection String</param>
		public SqlDataManager(string connString)
		{
			Connect(connString);
		}

		/// <summary>
		/// Avoid using this constructor unless there's a good reason to pass the SqlConnection
		/// </summary>
		/// <param name="connection">Connection object</param>
		public SqlDataManager(SqlConnection connection)
		{
			_sqlConnection = connection;
		}

		private void Connect(string connString)
		{
			_sqlConnection = new SqlConnection();
			_sqlConnection.ConnectionString = connString;
			_sqlConnection.Open();
		}

		/// <summary>
		/// Insert a record on the specified database table
		/// </summary>
		/// <param name="table">Table to insert the record to</param>
		/// <param name="values">Collection of values representing the actual record</param>
		/// <returns>Returns the identity value of the inserted row. Otherwise it retuns zero</returns>
		public int Insert(SqlTable table, ValueList values)
		{
			string sqlScript = "SELECT * FROM " + table.TableName + " WHERE 1=0 ";
			DataTable dt = new DataTable();
			SqlCommand command = new SqlCommand(sqlScript, _sqlConnection);
			SqlDataAdapter adapter = new SqlDataAdapter(command);
			adapter.Fill(dt);

			DataRow dr	= dt.NewRow();
			
			//assign values
			for (int i = 0; i < values.Count; i++)
			{
				//validate if the column exist on database
				if (dr.Table.Columns.Contains(values.GetKey(i)))
				{
					dr[values.GetKey(i)] = values[values.GetKey(i)];
				}
				
			}

			//insert row
			dt.Rows.Add(dr);
			SqlCommandBuilder scb	= new SqlCommandBuilder(adapter);
			adapter.InsertCommand	= scb.GetInsertCommand();
			adapter.RowUpdated += new SqlRowUpdatedEventHandler(adapter_RowUpdated);
			adapter.Update(dt);
			return(_identity);
		}

		private void adapter_RowUpdated(object sender, SqlRowUpdatedEventArgs e)
		{
			_identity	= 0;
			SqlCommand idCMD = new SqlCommand("SELECT @@IDENTITY", _sqlConnection);

			if (e.StatementType == StatementType.Insert)
			{
			  // Retrieve the identity value and store it in the CategoryID column.
				object o	= idCMD.ExecuteScalar();
				if (!(o is DBNull))
				{
					_identity = Convert.ToInt32(o);
				}
			}
		}

		/// <summary>
		/// Updates a particular record on the specified database table that matches the filter criteria provided by they keys argument.
		/// </summary>
		/// <param name="table">Table to update the record on</param>
		/// <param name="values">Collection of values representing the new values of the actual record</param>
		/// <param name="keys">Primary keys of the table along with the values to filter on</param>
		/// <returns>Returns the number of rows afected by the query</returns>
		public int Update(SqlTable table, ValueList values, ValueList keys)
		{
			//Generate sql script to retrieve the record to update
			string sqlScript	= "SELECT * FROM "+table.TableName+" WHERE 1=1 ";
			for (int i = 0; i < keys.Count; i++)
			{
				if (keys[i] is int)
				{
					sqlScript += " AND " + keys.GetKey(i) + " = " + keys[i];
				}
				else
				{
					sqlScript += " AND " + keys.GetKey(i) + " = '" + keys[i] + "'";
				}
			}

			DataTable dt = new DataTable();
			SqlCommand command = new SqlCommand(sqlScript, _sqlConnection);
			SqlDataAdapter adapter = new SqlDataAdapter(command);
			adapter.Fill(dt);

			//assign values
			for (int i = 0; i < values.Count; i++)
			{
				//validate if the column exist on database
				if (dt.Rows[0].Table.Columns.Contains(values.GetKey(i)))
				{
					//do not modify primary key values
					if (!keys.Contains(values.GetKey(i)))
					{
						dt.Rows[0][values.GetKey(i)] = values[values.GetKey(i)];
					}
				}
			}

			//update row
			SqlCommandBuilder scb = new SqlCommandBuilder(adapter);
			adapter.UpdateCommand	= scb.GetUpdateCommand();
			return adapter.Update(dt);
		}

		/// <summary>
		/// Executes a SQL script on the database and returns a DataTable containing the results (if any).
		/// </summary>
		/// <param name="sqlScript">SQL script to execute</param>
		/// <returns>DataTable containing the results of the execution of the script</returns>
		public DataTable ExecuteSQL(string sqlScript)
		{
			return ExecuteSQL(sqlScript, new List<SqlParameter>());
		}

		/// <summary>
		/// Alternative method to execute parametrised SQL scripts
		/// </summary>
		/// <param name="sqlScript"></param>
		/// <param name="parameterList"></param>
		/// <returns></returns>
		public DataTable ExecuteSQL(string sqlScript, List<SqlParameter> parameterList)
		{
			DataTable dt = new DataTable();
			SqlCommand sqlCommand = new SqlCommand(sqlScript, _sqlConnection);
			foreach (SqlParameter parameter in parameterList)
			{
				sqlCommand.Parameters.Add(parameter);
			}
			SqlDataAdapter sqlDataAdapter = new SqlDataAdapter(sqlCommand);
			sqlDataAdapter.Fill(dt);
			return dt;
		}

		#region helper methods
		/// <summary>
		/// Validates null and DBNull and returns default value for the given type
		/// </summary>
		/// <param name="o"></param>
		/// <returns></returns>
		public static string GetStringFromDB(object o)
		{
			return (o is DBNull || o == null) ? String.Empty : ((string) o).Trim();
		}

		/// <summary>
		/// Validates null and DBNull and returns default value for the given type
		/// </summary>
		/// <param name="o"></param>
		/// <returns></returns>
		public static double GetDoubleFromDB(object o)
		{
			return (o is DBNull || o == null) ? 0 : (double) o;
		}

		/// <summary>
		/// Validates null and DBNull and returns default value for the given type
		/// </summary>
		/// <param name="o"></param>
		/// <returns></returns>
		public static char GetCharFromDB(object o)
        {
            return (o is DBNull || o == null) ? ' ' : (char)o;
        }

		/// <summary>
		/// Validates null and DBNull and returns default value for the given type
		/// </summary>
		/// <param name="o"></param>
		/// <returns></returns>
		public static int GetIntFromDB(object o)
		{
			return (o is DBNull || o == null) ? 0 : (int) o;
		}

		/// <summary>
		/// Validates null and DBNull and returns default value for the given type
		/// </summary>
		/// <param name="o"></param>
		/// <returns></returns>
		public static DateTime GetDateTimeFromDB(object o)
		{
			return (o is DBNull || o == null) ? DateTime.MinValue : (DateTime) o;
		}

		/// <summary>
		/// Validates null and DBNull and returns default value for the given type
		/// </summary>
		/// <param name="o"></param>
		/// <returns></returns>
		public static bool GetBooleanFromDB(object o)
		{
			return (o is DBNull || o == null) ? false : (bool) o;
		}
		#endregion
	}
}
